package processing

import (
	"errors"

	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/resources/resource_model"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type processingMongoAccessor struct {
	utils.AbstractAccessor // AbstractAccessor contains the basic fields of an accessor (model, caller)
}

// New creates a new instance of the processingMongoAccessor
func New() *processingMongoAccessor {
	return &processingMongoAccessor{
		utils.AbstractAccessor{
			ResourceModelAccessor: resource_model.New(),
		},
	}
}

/*
* Nothing special here, just the basic CRUD operations
 */

func (pma *processingMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return pma.GenericDeleteOne(id, pma)
}

func (pma *processingMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	set.(*ProcessingResource).ResourceModel = nil
	return pma.GenericUpdateOne(set.(*ProcessingResource).Trim(), id, pma, &ProcessingResource{})
}

func (pma *processingMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	data.(*ProcessingResource).ResourceModel = nil
	return pma.GenericStoreOne(data.(*ProcessingResource).Trim(), pma)
}

func (pma *processingMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return pma.StoreOne(data)
}

func (pma *processingMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {

	var processing ProcessingResource

	res_mongo, code, err := mongo.MONGOService.LoadOne(id, pma.GetType())
	if err != nil {
		pma.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}

	res_mongo.Decode(&processing)
	if !processing.VerifyAuth(pma.PeerID, pma.Groups) {
		return nil, 403, errors.New("You are not allowed to access this collaborative area")
	}
	resources, _, err := pma.ResourceModelAccessor.Search(nil, pma.GetType())
	if err == nil && len(resources) > 0 {
		processing.ResourceModel = resources[0].(*resource_model.ResourceModel)
	}
	return &processing, 200, nil
}

func (wfa processingMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []ProcessingResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	resources, _, err := wfa.ResourceModelAccessor.Search(nil, wfa.GetType())
	for _, r := range results {
		if !r.VerifyAuth(wfa.PeerID, wfa.Groups) {
			continue
		}
		if err == nil && len(resources) > 0 {
			r.ResourceModel = resources[0].(*resource_model.ResourceModel)
		}
		objs = append(objs, &r) // only get the abstract resource !
	}
	return objs, 200, nil
}

// Search searches for processing resources in the database, given some filters OR a search string
func (wfa *processingMongoAccessor) Search(filters *dbs.Filters, search string) ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	if (filters == nil || len(filters.And) == 0 || len(filters.Or) == 0) && search != "" {
		filters = &dbs.Filters{
			Or: map[string][]dbs.Filter{ // filter by like name, short_description, description, owner, url if no filters are provided
				"abstractresource.abstractobject.name": {{Operator: dbs.LIKE.String(), Value: search}},
				"abstractresource.short_description":   {{Operator: dbs.LIKE.String(), Value: search}},
				"abstractresource.description":         {{Operator: dbs.LIKE.String(), Value: search}},
				"abstractresource.owner":               {{Operator: dbs.LIKE.String(), Value: search}},
				"abstractresource.source_url":          {{Operator: dbs.LIKE.String(), Value: search}},
			},
		}
	}
	res_mongo, code, err := mongo.MONGOService.Search(filters, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not store to db. Error: " + err.Error())
		return nil, code, err
	}
	var results []ProcessingResource
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	resources, _, err := wfa.ResourceModelAccessor.Search(nil, wfa.GetType())
	for _, r := range results {
		if !r.VerifyAuth(wfa.PeerID, wfa.Groups) {
			continue
		}
		if err == nil && len(resources) > 0 {
			r.ResourceModel = resources[0].(*resource_model.ResourceModel)
		}
		objs = append(objs, &r) // only get the abstract resource !
	}
	return objs, 200, nil
}
